@@ -1,4 +1,4 @@
-/* $Id: actions.c,v 1.38 2011/10/23 13:47:03 imilh Exp $ */
+/* $Id: actions.c,v 1.48 2012/05/28 10:56:27 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010, 2011 The NetBSD Foundation, Inc.
@@ -118,6 +118,7 @@ pkg_download(Plisthead *installhead)
 
 		/* if pkg's repo URL is file://, just symlink */
 		if (strncmp(pkg_url, SCHEME_FILE, strlen(SCHEME_FILE)) == 0) {
+			(void)unlink(pkg_fs);
 			if (symlink(&pkg_url[strlen(SCHEME_FILE) + 3], pkg_fs) < 0)
 				errx(EXIT_FAILURE, MSG_SYMLINK_FAILED, pkg_fs);
 			printf(MSG_SYMLINKING_PKG, pkg_url);
@@ -632,35 +633,91 @@ pkgin_remove(char **pkgargs)
 	return rc;
 }
 
-/* find closest match for packages to be upgraded */
+/* actually unused, not sure if really needed */
+char *
+read_preferred(char *pkgname)
+{
+	int		matchlen;
+	FILE	*fp;
+	char	match[BUFSIZ], *pref, *p;
+
+	if ((fp = fopen(PKGIN_CONF"/"PREFERRED_PKGS, "r")) == NULL)
+		return NULL;
+
+	snprintf(match, BUFSIZ, "%s=", pkgname);
+	matchlen = strlen(match);
+
+	XMALLOC(pref, BUFSIZ * sizeof(char));
+
+	while (fgets(pref, BUFSIZ, fp) != NULL) {
+		/* pref == "foobar=" */
+		if (strncmp(match, pref, matchlen) == 0) {
+			p = pref + matchlen;
+
+			for (; *p == ' ' || *p == '\t'; p++);
+
+			if (*p == '\0')
+				return NULL;
+
+			snprintf(pref, BUFSIZ, "%s-%s", pkgname, p);
+			pref[strlen(pref) - 1] = '\0';
+			return pref;
+		}
+	}
+
+	fclose(fp);
+
+	return NULL;
+}
+
+/* 
+ * find closest match for packages to be upgraded 
+ * if we have mysql-5.1.10 installed prefer mysql-5.1.20 over
+ * mysql-5.5.20 when upgrading
+ */
 static char *
-narrow_match(char *pkgname, const char *fullpkgname)
+narrow_match(Pkglist *opkg)
 {
 	Pkglist	*pkglist;
-	char	*best_match = NULL;
+	char	*best_match, *preferred;
 	unsigned int		i;
-	size_t  pkglen, fullpkglen, matchlen;
+	size_t  fullpkglen, r_fullpkglen, matchlen;
 
 	matchlen = 0;
-	pkglen = strlen(pkgname);
-	fullpkglen = strlen(fullpkgname);
+	fullpkglen = strlen(opkg->full);
+
+	preferred = read_preferred(opkg->name);
+	best_match = NULL;
 
 	SLIST_FOREACH(pkglist, &r_plisthead, next) {
-		if (strlen(pkglist->name) == pkglen &&
-			strncmp(pkgname, pkglist->name, pkglen) == 0) {
+		if (strcmp(opkg->name, pkglist->name) != 0)
+			continue;
 
-			for (i = 0;
-				 i < fullpkglen && fullpkgname[i] == pkglist->full[i];
-				i++);
+		/*
+		 * if PKGPATH does not match, do not try to update (mysql 5.1/5.5)
+		 */
+		if (strcmp(opkg->pkgpath, pkglist->pkgpath) != 0)
+				continue;
 
-			if (i > matchlen) {
-				matchlen = i;
-				XSTRDUP(best_match, pkglist->full);
-			}
+		r_fullpkglen = strlen(pkglist->full);
 
+		/* we already have his package, or installed version is greater */
+		if (strcmp(opkg->full, pkglist->full) >= 0)
+			continue;
+
+		/* match as long as we can */
+		for (i = 0; i < fullpkglen && i < r_fullpkglen &&
+				 opkg->full[i] == pkglist->full[i];
+			 i++);
+
+		if (i > matchlen) {
+			XFREE(best_match);
+			matchlen = i;
+			XSTRDUP(best_match, pkglist->full);
 		}
+
 	} /* SLIST_FOREACH remoteplisthead */
-	XFREE(pkgname);
+
 
 	return best_match;
 }
@@ -679,9 +736,7 @@ record_upgrades(Plisthead *plisthead)
 
 	count = 0;
 	SLIST_FOREACH(pkglist, plisthead, next) {
-		XSTRDUP(pkgargs[count], pkglist->name);
-
-		pkgargs[count] = narrow_match(pkgargs[count], pkglist->full);
+		pkgargs[count] = narrow_match(pkglist);
 
 		if (pkgargs[count] == NULL)
 			continue;
