@@ -1,4 +1,4 @@
-/* $Id: cmd.h,v 1.7 2011/10/23 13:22:41 imilh Exp $ */
+/* $Id: cmd.h,v 1.14 2012/05/28 10:56:27 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010, 2011 The NetBSD Foundation, Inc.
@@ -56,6 +56,12 @@ static struct command {
 	  PKG_SHFDP_CMD },
 	{ "show-rev-deps", "srd", "Display reverse dependencies recursively.",
 	  PKG_SHRDP_CMD },
+	{ "show-category", "sc", "Show packages belonging to category.",
+	  PKG_SHCAT_CMD },
+	{ "show-pkg-category", "spc", "Show package's category.",
+	  PKG_SHPCAT_CMD },
+	{ "show-all-categories", "sac", "Show all categories.",
+	  PKG_SHALLCAT_CMD },
 	{ "keep", "ke", "Mark package as \"non auto-removable\".",
 	  PKG_KEEP_CMD },
 	{ "unkeep", "uk", "Mark package as \"auto-removable\".",
@@ -76,6 +82,12 @@ static struct command {
 	  PKG_SHPROV_CMD },
 	{ "requires", "req", "Show what files a package requires.",
 	  PKG_SHREQ_CMD },
+	{ "pkg-content", "pc", "Show remote package's content.",
+	  PKG_SHPKGCONT_CMD },
+	{ "pkg-descr", "pd", "Show remote package's long-description.",
+	  PKG_SHPKGDESC_CMD },
+	{ "pkg-build-defs", "pbd", "Show remote package's build definitions.",
+	  PKG_SHPKGBDEFS_CMD },
 	{ "tonic", "to", "Gin Tonic recipe.",
 	  PKG_GINTO_CMD },
 	{ NULL, NULL, NULL, 0 }
