@@ -1,4 +1,4 @@
-/* $Id: messages.h,v 1.26 2011/10/29 11:50:22 imilh Exp $ */
+/* $Id: messages.h,v 1.29 2012/05/28 10:56:27 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -41,6 +41,7 @@
 #define MSG_PKG_ARGS_KEEP "specify at least one package to keep"
 #define MSG_PKG_ARGS_UNKEEP "specify at least one package to unkeep"
 #define MSG_MISSING_SRCH "missing search string"
+#define MSG_MISSING_CATEGORY "missing category"
 
 #define MSG_USAGE "Usage: %s [-cdfFhlnPtvVy] command [package ...]\n"
 #define MSG_CMDS_SHORTCUTS "\nCommands and shortcuts:\n"
@@ -145,6 +146,8 @@ please re-run %s with a package name matching one of the following:\n"
 /* pkglist.c */
 #define MSG_IS_INSTALLED_CODE "\n=: package is installed and up-to-date\n<: package is installed but newer version is available\n>: installed package has a greater version than available package\n"
 #define MSG_NO_SEARCH_RESULTS "No results found for %s\n"
+#define MSG_EMPTY_LIST "Requested list is empty.\n"
+#define MSG_NO_CATEGORIES "No categories found.\n"
 
 /* fsops.c */
 #define MSG_TRANS_FAILED "Failed to translate %s in repository config file"
