@@ -1,7 +1,7 @@
-/* $Id: order.c,v 1.1.1.1 2011/03/03 14:43:12 imilh Exp $ */
+/* $Id: order.c,v 1.6 2011/10/02 14:54:12 imilh Exp $ */
 
 /*
- * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010, 2011 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -32,20 +32,22 @@
 
 #include "pkgin.h"
 
-/*
+/**
+ * /file order.c
+ *
  * order.c has only one purpose: arrange lists order in order to satisfy
  * pkg_add / pkg_delete.
  */
 
-/* find dependency deepness for package removal and record it to pdp->level */
+/**
+ * find dependency deepness for package removal and record it to pdp->level
+ */
 static void
-remove_dep_deepness(Deptreehead *deptreehead)
+remove_dep_deepness(Plisthead *deptreehead)
 {
-	char *depname;
-	Pkgdeptree *pdp;
-	Deptreehead lvldeptree;
-
-	SLIST_INIT(&lvldeptree);
+	char		*depname;
+	Pkglist		*pdp;
+	Plisthead	*lvldeptree;
 
 	/* get higher recursion level */
 	SLIST_FOREACH(pdp, deptreehead, next) {
@@ -56,38 +58,43 @@ remove_dep_deepness(Deptreehead *deptreehead)
 
 		pdp->level = 1;
 		
-		if (pdp->depname == NULL)
+		if (pdp->depend == NULL)
 			/* there's something wrong with database's record, probably
 			 * a mistaken dependency
 			 */
 			continue;
 
 		/* depname received from deptreehead is in package format */
-		XSTRDUP(depname, pdp->depname);
+		XSTRDUP(depname, pdp->depend);
 
 		trunc_str(depname, '-', STR_BACKWARD);
 
-		full_dep_tree(depname, LOCAL_REVERSE_DEPS, &lvldeptree);
+		lvldeptree = init_head();
+		full_dep_tree(depname, LOCAL_REVERSE_DEPS, lvldeptree);
 
-		if (!SLIST_EMPTY(&lvldeptree))
-		    	pdp->level = SLIST_FIRST(&lvldeptree)->level + 1;
+		if (!SLIST_EMPTY(lvldeptree))
+		    	pdp->level = SLIST_FIRST(lvldeptree)->level + 1;
 
 		XFREE(depname);
-		free_deptree(&lvldeptree);
+		free_pkglist(&lvldeptree, DEPTREE);
 
 #if 0
-		printf("%s -> %d\n", pdp->depname, pdp->level);
+		printf("%s -> %d\n", pdp->depend, pdp->level);
 #endif
 	}
 }
 
-/* order the remove list according to dependency level */
-Deptreehead *
-order_remove(Deptreehead *deptreehead)
+/**
+ * \fn order_remove
+ *
+ * \brief order the remove list according to dependency level
+ */
+Plisthead *
+order_remove(Plisthead *deptreehead)
 {
-	int i, maxlevel = 0;
-	Pkgdeptree *pdp, *next;
-	Deptreehead *ordtreehead;
+	int			i, maxlevel = 0;
+	Pkglist		*pdp, *next;
+	Plisthead	*ordtreehead;
 
 	/* package removal cannot trust recorded dependencies, reorder */
 	remove_dep_deepness(deptreehead);
@@ -96,15 +103,14 @@ order_remove(Deptreehead *deptreehead)
 		if (pdp->level > maxlevel)
 			maxlevel = pdp->level;
 
-	XMALLOC(ordtreehead, sizeof(Deptreehead));
-	SLIST_INIT(ordtreehead);
+	ordtreehead = init_head();
 
 	for (i = maxlevel; i >= 0; i--) {
 		pdp = SLIST_FIRST(deptreehead);
 		while (pdp != NULL) {
 			next = SLIST_NEXT(pdp, next);
 			if (pdp->level == i) {
-				SLIST_REMOVE(deptreehead, pdp, Pkgdeptree, next);
+				SLIST_REMOVE(deptreehead, pdp, Pkglist, next);
 				SLIST_INSERT_HEAD(ordtreehead, pdp, next);
 			}
 			pdp = next;
@@ -116,22 +122,17 @@ order_remove(Deptreehead *deptreehead)
 
 /* find dependency deepness for upgrade and record it to pimpact->level */
 static void
-upgrade_dep_deepness(Impacthead *impacthead)
+upgrade_dep_deepness(Plisthead *impacthead)
 {
-	char		*depname, *p;
-	Pkgimpact	*pimpact;
-	Deptreehead	lvldeptree;
-	Plisthead	*plisthead;
-
-	plisthead = rec_pkglist(LOCAL_PKGS_QUERY);
-
-	SLIST_INIT(&lvldeptree);
+	char		*pkgname;
+	Pkglist		*pimpact;
+    Plisthead	*lvldeptree;
 
 	/* get higher recursion level */
 	SLIST_FOREACH(pimpact, impacthead, next) {
 		if (pimpact->level == -1) { /* unique package, just return */
 			pimpact->level = 0;
-			goto endupdep;
+			return;
 		}
 
 		/* only deal with TOUPGRADE and TOREMOVE */
@@ -140,37 +141,39 @@ upgrade_dep_deepness(Impacthead *impacthead)
 
 		pimpact->level = 1;
 
-		/* depname received from impact is in real dependency format */
-		depname = end_expr(plisthead, pimpact->depname);
+		/* depname received from impact is in full package format */
+		XSTRDUP(pkgname, pimpact->full);
 
-		if ((p = strrchr(depname, DELIMITER)) != NULL)
-			*p = '\0';
+		trunc_str(pkgname, '-', STR_BACKWARD);
 
-		full_dep_tree(depname, LOCAL_REVERSE_DEPS, &lvldeptree);
+		lvldeptree = init_head();
+		full_dep_tree(pkgname, LOCAL_REVERSE_DEPS, lvldeptree);
 
-		if (!SLIST_EMPTY(&lvldeptree))
-		    	pimpact->level = SLIST_FIRST(&lvldeptree)->level + 1;
+		if (!SLIST_EMPTY(lvldeptree))
+		    	pimpact->level = SLIST_FIRST(lvldeptree)->level + 1;
 
 #if 0
-		printf("%s (%s) -> %d\n", pimpact->depname, depname, pimpact->level);
+		printf("%s (%s) -> %d\n",
+			pimpact->full, pkgname, pimpact->level);
 #endif
 
-		XFREE(depname);
-		free_deptree(&lvldeptree);
+		XFREE(pkgname);
+		free_pkglist(&lvldeptree, DEPTREE);
 	}
-
-endupdep:
-	free_pkglist(plisthead);
 }
 
-/* order the remove-for-upgrade list according to dependency level */
-Deptreehead *
-order_upgrade_remove(Impacthead *impacthead)
+uint8_t pi_upgrade = 0;
+/**
+ * \fn order_upgrade_remove
+ *
+ * \brief order the remove-for-upgrade list according to dependency level
+ */
+Plisthead *
+order_upgrade_remove(Plisthead *impacthead)
 {
-	Deptreehead *ordtreehead;
-	Pkgimpact *pimpact;
-	Pkgdeptree *pdp;
-	int i, maxlevel = 0;
+	Plisthead	*ordtreehead;
+	Pkglist		*pimpact, *pdp;
+	int			i, maxlevel = 0;
 
 	upgrade_dep_deepness(impacthead);
 
@@ -180,38 +183,45 @@ order_upgrade_remove(Impacthead *impacthead)
 				&& pimpact->level > maxlevel)
 			maxlevel = pimpact->level;
 
-	XMALLOC(ordtreehead, sizeof(Deptreehead));
-	SLIST_INIT(ordtreehead);
+	ordtreehead = init_head();
 
 	for (i = maxlevel; i >= 0; i--)
 		SLIST_FOREACH(pimpact, impacthead, next) {
 			if ((pimpact->action == TOUPGRADE ||  pimpact->action == TOREMOVE)
 				&& pimpact->level == i) {
 
-				XMALLOC(pdp, sizeof(Pkgdeptree));
-				XSTRDUP(pdp->depname, pimpact->oldpkg);
-				pdp->matchname = NULL; /* safety */
+				if (pkg_in_impact(ordtreehead, pimpact->old))
+					continue;
+
+				pdp = malloc_pkglist(DEPTREE);
+
+				XSTRDUP(pdp->depend, pimpact->old);
+				pdp->name = NULL; /* safety */
 				pdp->computed = pimpact->action; /* XXX: ugly */
+				pdp->level = pimpact->level; /* informative only */
 				SLIST_INSERT_HEAD(ordtreehead, pdp, next);
-			}
-		}
+
+			} /* action == TOUPGRADE || TOREMOVE */
+		} /* for maxlevel */
 
 	return ordtreehead;
 }
 
-/*
+/**
+ * \fn order_install
+ *
  * order the install list according to dependency level
  * here we only rely on basic level given by pkg_summary, the only drawback
  * is that pkg_add will install direct dependencies, giving a "failed,
  * package already installed"
  */
-Deptreehead *
-order_install(Impacthead *impacthead)
+Plisthead *
+order_install(Plisthead *impacthead)
 {
-	Deptreehead *ordtreehead;
-	Pkgimpact *pimpact;
-	Pkgdeptree *pdp;
-	int i, maxlevel = 0;
+	Plisthead	*ordtreehead;
+	Pkglist		*pimpact, *pdp, *pi_dp = NULL;
+	int			i, maxlevel = 0;
+	char		tmpcheck[BUFSIZ];
 
 	/* record higher dependency level on impact list */
 	SLIST_FOREACH(pimpact, impacthead, next) {
@@ -220,24 +230,41 @@ order_install(Impacthead *impacthead)
 			maxlevel = pimpact->level;
 	}
 
-	XMALLOC(ordtreehead, sizeof(Deptreehead));
-	SLIST_INIT(ordtreehead);
+	ordtreehead = init_head();
 
 	for (i = 0; i <= maxlevel; i++) {
 		SLIST_FOREACH(pimpact, impacthead, next) {
 			if ((pimpact->action == TOUPGRADE ||
 					pimpact->action == TOINSTALL) && pimpact->level == i) {
-				XMALLOC(pdp, sizeof(Pkgdeptree));
-				XSTRDUP(pdp->depname, pimpact->pkgname);
-				pdp->matchname = NULL; /* safety */
+
+				if (pkg_in_impact(ordtreehead, pimpact->full))
+					continue;
+
+				pdp = malloc_pkglist(DEPTREE);
+
+				XSTRDUP(pdp->depend, pimpact->full);
+				pdp->name = NULL; /* safety */
 				pdp->level = pimpact->level;
 				/* record package size for download check */
 				pdp->file_size = pimpact->file_size;
 
-				SLIST_INSERT_HEAD(ordtreehead, pdp, next);
-			}
-		}
-	}
+				/* check for pkg_install upgrade */
+				strcpy(tmpcheck, pimpact->full);
+				trunc_str(tmpcheck, '-', STR_BACKWARD);
+				/* match on pkg_install */
+				if (strcmp(tmpcheck, PKG_INSTALL) == 0) {
+					pi_upgrade = 1;
+					/* backup pdp for future insertion */
+					pi_dp = pdp;
+				} else					
+					SLIST_INSERT_HEAD(ordtreehead, pdp, next);
+			} /* action == TOINSTALL */
+		} /* SLIST_FOREACH pimpact */
+	} /* for i < maxlevel */
+
+	/* pkg_install is to be upgraded, make it first */
+	if (pi_upgrade && pi_dp != NULL)
+		SLIST_INSERT_HEAD(ordtreehead, pi_dp, next);
 
 	return ordtreehead;
 }
