@@ -1,4 +1,4 @@
-/* $Id: pkgin.h,v 1.28 2011/10/23 13:47:03 imilh Exp $ */
+/* $Id: pkgin.h,v 1.36 2012/05/28 10:56:27 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -56,6 +56,7 @@
 #endif
 #define PKG_DELETE PKGTOOLS"/pkg_delete"
 #define PKG_ADD PKGTOOLS"/pkg_add"
+#define PKG_INFO PKGTOOLS"/pkg_info"
 
 #define PKG_SUMMARY "pkg_summary"
 #define PKGIN_SQL_LOG PKGIN_DB"/sql.log"
@@ -65,6 +66,7 @@
 #define PKGIN_CONF PKG_SYSCONFDIR"/pkgin"
 #define REPOS_FILE "repositories.conf"
 #define PKG_INSTALL "pkg_install"
+#define PREFERRED_PKGS "preferred.conf"
 
 #define LOCAL_SUMMARY 0
 #define REMOTE_SUMMARY 1
@@ -109,6 +111,12 @@
 #define PKG_IMPORT_CMD 17
 #define PKG_SHPROV_CMD 18
 #define PKG_SHREQ_CMD 19
+#define PKG_SHPKGCONT_CMD 20
+#define PKG_SHPKGDESC_CMD 21
+#define PKG_SHPKGBDEFS_CMD 22
+#define PKG_SHCAT_CMD 23
+#define PKG_SHPCAT_CMD 24
+#define PKG_SHALLCAT_CMD 25
 #define PKG_GINTO_CMD 255
 
 #define PKG_EQUAL '='
@@ -165,6 +173,8 @@ typedef struct Pkglist {
 					foo>=1.0
 					or full package name for reverse dependencies:
 					foo-1.0 */
+	char *category; /*!< package category */
+	char *pkgpath; /*!< pkgsrc pkgpath */
 	union {
 		char		*comment; /*!< package list comment */
 		Deptree		deptree; /*<! dependency tree informations */
@@ -198,6 +208,7 @@ extern char			**pkg_repos;
 extern const char	*pkgin_cache;
 extern char  		lslimit;
 extern char			pkgtools_flags[];
+extern char			pkg_dbdir[];
 extern Plisthead	r_plisthead;
 extern Plisthead	l_plisthead;
 extern FILE			*tracefp;
@@ -225,6 +236,9 @@ Plisthead	*init_head(void);
 Plisthead	*rec_pkglist(const char *, ...);
 void		list_pkgs(const char *, int);
 void		search_pkg(const char *);
+void		show_category(char *);
+void		show_pkg_category(char *);
+void		show_all_categories(void);
 /* actions.c */
 int			check_yesno(uint8_t);
 void		do_pkg_remove(Plisthead *);
@@ -232,6 +246,7 @@ int			pkgin_remove(char **);
 int			pkgin_install(char **, uint8_t);
 char		*action_list(char *, char *);
 void		pkgin_upgrade(int);
+char		*read_preferred(char *);
 /* order.c */
 Plisthead	*order_remove(Plisthead *);
 Plisthead	*order_upgrade_remove(Plisthead *);
@@ -264,5 +279,9 @@ void		import_keep(uint8_t, const char *);
 int			pkg_met_reqs(Plisthead *);
 int			pkg_has_conflicts(Pkglist *);
 void		show_prov_req(const char *, const char *);
+/* pkg_infos.c */
+void		show_pkg_info(char, char *);
+/* pkgindb.c */
+void		get_pkg_dbdir(void);
 
 #endif
