@@ -1,7 +1,7 @@
-/* $Id: pkgindb_queries.c,v 1.20 2011/10/30 18:02:12 imilh Exp $ */
+/* $Id: pkgindb_queries.c,v 1.26 2012/05/28 10:56:27 imilh Exp $ */
 
 /*
- * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010, 2011, 2012 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -30,7 +30,7 @@
  *
  */
 
-const char const DROP_LOCAL_TABLES[] =
+const char DROP_LOCAL_TABLES[] =
     "DROP TABLE IF EXISTS LOCAL_DEPS;"
     "DROP TABLE IF EXISTS LOCAL_PKG;"
     "DROP TABLE IF EXISTS LOCAL_CONFLICTS;"
@@ -118,21 +118,39 @@ const char KEEP_PKG[] =
 const char UNKEEP_PKG[] =
     "UPDATE LOCAL_PKG SET PKG_KEEP = NULL WHERE PKGNAME = \'%s\';";
 
-const char LOCAL_PKGS_QUERY[] =
-    "SELECT FULLPKGNAME,PKGNAME,PKGVERS,COMMENT,FILE_SIZE,SIZE_PKG "
+/* for upgrades, prefer higher versions to be at the top of SLIST */
+const char LOCAL_PKGS_QUERY_ASC[] =
+    "SELECT FULLPKGNAME,PKGNAME,PKGVERS,"
+	"COMMENT,FILE_SIZE,SIZE_PKG,CATEGORIES,PKGPATH "
 	"FROM LOCAL_PKG "
     "ORDER BY FULLPKGNAME ASC;";
 
-const char REMOTE_PKGS_QUERY[] =
-    "SELECT FULLPKGNAME,PKGNAME,PKGVERS,COMMENT,FILE_SIZE,SIZE_PKG "
+const char REMOTE_PKGS_QUERY_ASC[] =
+    "SELECT FULLPKGNAME,PKGNAME,PKGVERS,"
+	"COMMENT,FILE_SIZE,SIZE_PKG,CATEGORIES,PKGPATH "
 	"FROM REMOTE_PKG "
     "ORDER BY FULLPKGNAME ASC;";
 
+/* for displays, prefer lower versions to be at the top of SLIST*/
+const char LOCAL_PKGS_QUERY_DESC[] =
+    "SELECT FULLPKGNAME,PKGNAME,PKGVERS,"
+	"COMMENT,FILE_SIZE,SIZE_PKG,CATEGORIES,PKGPATH "
+	"FROM LOCAL_PKG "
+    "ORDER BY FULLPKGNAME DESC;";
+
+const char REMOTE_PKGS_QUERY_DESC[] =
+    "SELECT FULLPKGNAME,PKGNAME,PKGVERS,"
+	"COMMENT,FILE_SIZE,SIZE_PKG,CATEGORIES,PKGPATH "
+	"FROM REMOTE_PKG "
+    "ORDER BY FULLPKGNAME DESC;";
+
 const char NOKEEP_LOCAL_PKGS[] =
-    "SELECT FULLPKGNAME,PKGNAME FROM LOCAL_PKG WHERE PKG_KEEP IS NULL;";
+    "SELECT FULLPKGNAME,PKGNAME,PKGPATH "
+	"FROM LOCAL_PKG WHERE PKG_KEEP IS NULL;";
 
 const char KEEP_LOCAL_PKGS[] =
-    "SELECT FULLPKGNAME,PKGNAME FROM LOCAL_PKG WHERE PKG_KEEP IS NOT NULL;";
+    "SELECT FULLPKGNAME,PKGNAME,PKGPATH "
+	"FROM LOCAL_PKG WHERE PKG_KEEP IS NOT NULL;";
 
 const char PKG_URL[] =
     "SELECT REPOSITORY FROM REMOTE_PKG WHERE FULLPKGNAME = \'%s\';";
@@ -179,3 +197,7 @@ const char GET_ORPHAN_PACKAGES[] =
 
 const char COMPAT_CHECK[] =
 	"SELECT FULLPKGNAME FROM REMOTE_PKG LIMIT 1;";
+
+const char SHOW_ALL_CATEGORIES[] =
+	"SELECT DISTINCT CATEGORIES FROM REMOTE_PKG WHERE "
+	"CATEGORIES NOT LIKE '%% %%' ORDER BY CATEGORIES DESC;";
